package cornell.ece4760.timesystem;

import android.app.Activity;
import android.app.ListActivity;
import android.content.ContentValues;
import android.content.Intent;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.Toast;

/**
 * This class creates the list of instructions the user
 * can add to the routine. The appropriate byte is added
 * to the routine. If more user input is required (loop,
 * jump) then prompt for user input.
 * 
 * @author Andy Liang
 */
public class Instruction extends ListActivity {
	// Debugging
	private static final String TAG = "Instruction";
	private String[] instructions;
	private int RoutineID=0;
	private final String dbName= "RoutineDB"	;
	private final String tableName= "RoutineContent";

	/**
	 * Get the current Routine ID and set up the instructions list.
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		RoutineID= getIntent().getExtras().getInt("RoutineID");
		instructions= getResources().getStringArray(R.array.spinner_instructions);
		this.setListAdapter(new ArrayAdapter<String>(this,
				android.R.layout.simple_list_item_1, instructions));

	}

	/**
	 * Method gets called when instruction tapped
	 */
	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		// TODO Auto-generated method stub
		//super.onListItemClick(l, v, position, id);
		// prepare byte
		Intent serverIntent;
		int byte1= 0;
		String label= "";
		switch(position) {
		case 0: // loop, prompt for number of loops and display remaining or not; 2 entries: header and back
			serverIntent = new Intent(this, DialogActivity.class);
			serverIntent.putExtra("InstID", 9);
			startActivityForResult(serverIntent, 9);
			break;
		case 1: // jump
			serverIntent = new Intent(this, DialogActivity.class);
			serverIntent.putExtra("InstID", 1);
			startActivityForResult(serverIntent, 1);
			break;
		case 2: // pause
			byte1= 133;
			label= "Pause";
			break;
		case 3: // ms timer
			byte1= 144;
			label= "Millisecond timer";
			break;
		case 4: // sched start
			byte1= 132;
			label= "Scheduled start";
			break;
		case 5: // display clk
			byte1= 129;
			label= "Display clock";
			break;
		case 6: // set clk
			byte1= 130;
			label= "Set clock";
			break;
		case 7:// screen demo
			byte1= 136;
			label= "Screen demo";
			break; 
		default:

			break;	

		}

		// insert appropriate byte1 for all instructions except loop and jump
		if (position > 1) {
			SQLiteDatabase  database= this.openOrCreateDatabase(dbName, MODE_PRIVATE, null);
			// get max Seq
			String rid= String.format("%d", RoutineID);
			Cursor c = database.rawQuery("SELECT Seq FROM " +
					tableName +
					" WHERE RoutineID = " + rid + " ORDER BY Seq DESC LIMIT 1", null);

			c.moveToFirst();
			int MaxSeq= c.getInt(c.getColumnIndex("Seq"));
			c.close();

			ContentValues updateRoutine = new ContentValues();
			updateRoutine.put("RoutineID", RoutineID);
			updateRoutine.put("Seq", MaxSeq+1);
			updateRoutine.put("Byte1", byte1);
			updateRoutine.put("Byte2", 0);
			updateRoutine.put("Label", label);
			updateRoutine.put("Message", "");
			try {
				long rowID = database.insertOrThrow(tableName, null, updateRoutine);
				String s= String.format("Inserted rowId:%d",rowID);
				Log.e(TAG,s);
			} catch (SQLException e) {
				String error= String.format("Error: %s", e.toString());
				Toast.makeText(this, error, Toast.LENGTH_SHORT).show();
				Log.e(TAG,error);
			}
			database.close();
			finish();
		} 
	}


	/**
	 * Method is called when the activities that ask for further user input
	 * is finished and inserts the instruction to the database accordingly.
	 */
	@Override 
	public void onActivityResult(int requestCode, int resultCode, Intent data) {     
		super.onActivityResult(requestCode, resultCode, data); 
		switch(requestCode) { 
		case (1) : { 
			if (resultCode == Activity.RESULT_OK) { 
				int dest = data.getIntExtra("RoutineID",1);
				// Update database
				SQLiteDatabase  database= this.openOrCreateDatabase(dbName, MODE_PRIVATE, null);
				// get max Seq
				String rid= String.format("%d", RoutineID);
				Cursor c = database.rawQuery("SELECT Seq FROM " +
						tableName +
						" WHERE RoutineID = " + rid + " ORDER BY Seq DESC LIMIT 1", null);

				c.moveToFirst();
				int MaxSeq= c.getInt(c.getColumnIndex("Seq"));
				c.close();

				ContentValues updateRoutine = new ContentValues();
				updateRoutine.put("RoutineID", RoutineID);
				updateRoutine.put("Seq", MaxSeq+1);
				updateRoutine.put("Byte1", 224+dest-1);
				updateRoutine.put("Byte2", 0);
				String label= String.format("Jump to Routine %d", dest);
				updateRoutine.put("Label", label);
				updateRoutine.put("Message", "");
				try {
					long rowID = database.insertOrThrow(tableName, null, updateRoutine);
					String s= String.format("Inserted rowId:%d",rowID);
					Log.e(TAG,s);
				} catch (SQLException e) {
					String error= String.format("Error: %s", e.toString());
					Toast.makeText(this, error, Toast.LENGTH_SHORT).show();
					Log.e(TAG,error);
				}
				database.close();
				finish();
			} 
			break; 
		} 
		case (9) : {
			if (resultCode == Activity.RESULT_OK) { 
				int loops = data.getIntExtra("Loops",1);
				boolean display = data.getBooleanExtra("Display", false);
				boolean inner = data.getBooleanExtra("Inner", false);
				// Update database
				SQLiteDatabase  database= this.openOrCreateDatabase(dbName, MODE_PRIVATE, null);
				// get max Seq
				String rid= String.format("%d", RoutineID);
				Cursor c = database.rawQuery("SELECT Seq FROM " +
						tableName +
						" WHERE RoutineID = " + rid + " ORDER BY Seq DESC LIMIT 1", null);

				c.moveToFirst();
				int MaxSeq= c.getInt(c.getColumnIndex("Seq"));
				c.close();

				// add header
				ContentValues updateRoutine = new ContentValues();
				updateRoutine.put("RoutineID", RoutineID);
				updateRoutine.put("Seq", MaxSeq+1);
				if (!inner) {
					updateRoutine.put("Byte1", 192+loops-1);
				} else {
					updateRoutine.put("Byte1", 160+loops-1);
				}
				updateRoutine.put("Byte2", 0);
				String label= (inner)?String.format("Inner loop x%d", loops):String.format("Outer loop x%d", loops);
				updateRoutine.put("Label", label);
				updateRoutine.put("Message", "");
				// add footer
				ContentValues updateRoutine2 = new ContentValues();
				updateRoutine2.put("RoutineID", RoutineID);
				updateRoutine2.put("Seq", MaxSeq+2);
				String label2= "End of ";
				if (display) {
					updateRoutine2.put("Byte1", 137);
					if (!inner) {
						updateRoutine2.put("Byte2", 134);
						label2 = label2 + String.format("outer loop, display remaining cycles",loops);
					} else {
						updateRoutine2.put("Byte2", 135);
						label2 = label2 + String.format("inner loop, display remaining cycles",loops);
					}

				} else {
					if (!inner) {
						updateRoutine2.put("Byte1", 134);
						label2= label2 + String.format("outer loop",loops);
					} else {
						updateRoutine2.put("Byte1", 135);
						label2= label2 + String.format("inner loop",loops);
					}
					updateRoutine2.put("Byte2", 0);
				}
				updateRoutine2.put("Label", label2);
				updateRoutine2.put("Message", "");
				try {
					database.insertOrThrow(tableName, null, updateRoutine);
					database.insertOrThrow(tableName, null, updateRoutine2);
				} catch (SQLException e) {
					String error= String.format("Error: %s", e.toString());
					Toast.makeText(this, error, Toast.LENGTH_SHORT).show();
					Log.e(TAG,error);
				}
				database.close();
				finish();
			} 
			break; 
		} 
		}
	} 
}

