package cornell.ece4760.timesystem;

import java.util.ArrayList;

import android.app.Activity;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.graphics.Color;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;


/**
 * Activity that controls routine instruction display and manipulation
 * of the instructions.
 * 
 * @author Andy Liang
 */
public class RoutineConfig extends Activity {
	
	private static final String TAG= "RoutineConfig";
	
	private ArrayList<String> tasks;
	public static ArrayList<Integer> seqArray;
	private int RoutineID= 0;
	private SQLiteDatabase database= null;
	private final String dbName= "RoutineDB";
	private final String tableName= "RoutineContent";
	private myAdapter adapter;
	private ListView listView= null;
	
	private static class myAdapter extends BaseAdapter {
		private LayoutInflater mInflater;
		private ArrayList<String> tasks;

		public myAdapter(Context context) {
			mInflater = LayoutInflater.from(context);

		}
		public void setTasks(ArrayList<String> mTasks) {
			tasks= mTasks;
		}
		
		public int getCount() {
			return tasks.size();
		}

		public Object getItem(int position) {
			return position;
		}

		public long getItemId(int position) {
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			ViewHolder holder;
			if (convertView == null) {
				convertView = mInflater.inflate(R.layout.instruction_list_item, null);
				holder = new ViewHolder();
				holder.textview = (TextView) convertView
				.findViewById(R.id.textview);

				convertView.setTag(holder);
				
			} else {
				holder = (ViewHolder) convertView.getTag();
			}
			if (tasks.get(position).indexOf("uter loop") > -1) {
				holder.textview.setTextColor(Color.GREEN);
			} else if (tasks.get(position).indexOf("nner loop") > -1) {
				holder.textview.setTextColor(Color.CYAN);
			}
			holder.textview.setText(tasks.get(position));
			
			return convertView;
		}
	

		static class ViewHolder {
			TextView textview;
		}
		
		
	}


	  @Override
	    public void onCreate(Bundle savedInstanceState) {
	        super.onCreate(savedInstanceState);

	        // get current RoutineID
	        RoutineID= getIntent().getExtras().getInt("RoutineID");
	        setContentView(R.layout.routine_config);
	        listView= (ListView) findViewById(R.id.config_list);
	        listView.setOnItemClickListener(new OnItemClickListener() {
	        	@Override
	        	public void onItemClick(AdapterView arg0, View arg1, int position, long arg3) {
	        		selected(position);
	        	}
	        });
	        listView.setOnItemLongClickListener(new OnItemLongClickListener() {
	        	@Override
	        	public boolean onItemLongClick(AdapterView<?>av, View v, int pos, long id) {
	        		longClicked(pos);
	        		return true;
	        	}
	        });
	        refreshList();

	  }

	  private void refreshList() {
		  tasks= new ArrayList<String>();
		  seqArray= new ArrayList<Integer>();
		 
	        
	        database= this.openOrCreateDatabase(dbName, MODE_PRIVATE, null);
	        
	        String rid= String.format("%d", RoutineID);
	        Cursor c = database.rawQuery("SELECT * FROM " +
	    			tableName +
	    			" WHERE RoutineID = " + rid + " ORDER BY Seq ASC", null);
	        
	        if (c != null ) {
      		if  (c.moveToFirst()) {
      			do {
      				String b= c.getString(c.getColumnIndex("Label"));
      				int i= c.getInt(c.getColumnIndex("Seq"));
      				seqArray.add(i);
      				String m= c.getString(c.getColumnIndex("Message"));
      				if (m.length() != 0) {
      					tasks.add(b+" : "+m);
      				} else {
      					tasks.add(b);
      				}
      				
      			}while (c.moveToNext());
      		} 
      	}
	        c.close();
	        database.close();
	        adapter= new myAdapter(this);
	        adapter.setTasks(tasks);
	        listView.setAdapter(adapter);


	  }
	  
	  @Override
	    public void onStart() {
	        super.onStart();
	        Log.e(TAG, "++ ON START ++");
	       
	  }
	  
	  @Override
	    public void onResume() {
	        super.onResume();
	        Log.e(TAG, "++ ON Resume ++");
	        refreshList();
	  }
	  
	  @Override
	  public void onStop() {
	        super.onStop();
	        Log.e(TAG, "++ ON Pause ++");
	  }
	  
	  private void longClicked(int position) {
		  if ((tasks.get(position).indexOf("\u2191") != -1) || (tasks.get(position).indexOf("\u2193") != -1)) {
			  Intent serverIntent = new Intent(this, Timing.class);
			  serverIntent.putExtra("Seq", seqArray.get(position));
			  serverIntent.putExtra("RoutineID", RoutineID);
			  serverIntent.putExtra("Min", Integer.parseInt(tasks.get(position).substring(2,4)));
			  serverIntent.putExtra("Sec", Integer.parseInt(tasks.get(position).substring(5,7)));
			  serverIntent.putExtra("UD", (tasks.get(position).indexOf("\u2191") != -1));
			  if (tasks.get(position).length() > 10) {
				  serverIntent.putExtra("Message", tasks.get(position).substring(10));
			  } else {
				  serverIntent.putExtra("Message", "");
			  }
			  startActivityForResult(serverIntent, 1);
		  }
	  }
	  
	  private void selected(int position) {
		  System.out.println("selected");
		  // TODO Auto-generated method stub
		  //super.onListItemClick(l, v, position, id);
		  if (seqArray.get(position) == 0) {
			  Toast.makeText(this, "Routine Header can not be moved", Toast.LENGTH_LONG).show();
		  } else {
			  Intent serverIntent = new Intent(this, DialogActivity.class);
			  serverIntent.putExtra("Seq", seqArray.get(position));
			  serverIntent.putExtra("InstID", 5);
			  startActivityForResult(serverIntent, 5);
		  }

	  }
	  
	  @Override 
	  public void onActivityResult(int requestCode, int resultCode, Intent data) {     
		  super.onActivityResult(requestCode, resultCode, data); 
		  switch(requestCode) { 
		  case (5) : { 
			  if (resultCode == Activity.RESULT_OK) { 
				  int seq = data.getIntExtra("Seq",1);
				  int dir = data.getIntExtra("Direction", 0);
				  // swap row Seq based on direction
				  database= this.openOrCreateDatabase(dbName, MODE_PRIVATE, null);
				  

				  String rid= String.format("%d", RoutineID);
				  String sequence= String.format("%d", seq);
				  if (dir == 1) { // move up
					
					  if (seqArray.indexOf(seq) > 1) { // not trying to swap with header
						  String swapSeq= String.format("%d",seqArray.get(seqArray.indexOf(seq)-1));

						  ContentValues vals = new ContentValues();
						  vals.put("Seq", 99999);
						  database.update(tableName, vals, "RoutineID = " + rid + " AND Seq = " + sequence, null);
						  vals.clear();

						  vals.put("Seq", seq);
						  database.update(tableName, vals, "RoutineID = " + rid + " AND Seq = " + swapSeq, null);
						  vals.clear();

						  vals.put("Seq", swapSeq);
						  database.update(tableName, vals, "RoutineID = " + rid + " AND Seq = 99999", null);
						  vals.clear();

					  } else {
						  Toast.makeText(this, "Item can not be moved up", Toast.LENGTH_LONG).show();
					  }
				  } else if (dir == 2) { // move down
					  
					  if ((seqArray.indexOf(seq) < seqArray.size()-1)) { // not trying to move footer down too far
						  String swapSeq= String.format("%d",seqArray.get(seqArray.indexOf(seq)+1));
						  ContentValues vals = new ContentValues();
						  vals.put("Seq", 99999);
						  database.update(tableName, vals, "RoutineID = " + rid + " AND Seq = " + sequence, null);
						  vals.clear();

						  vals.put("Seq", seq);
						  database.update(tableName, vals, "RoutineID = " + rid + " AND Seq = " + swapSeq, null);
						  vals.clear();

						  vals.put("Seq", swapSeq);
						  database.update(tableName, vals, "RoutineID = " + rid + " AND Seq = 99999", null);
						  vals.clear();
					  } else {
						  Toast.makeText(this, "Item can not be moved down", Toast.LENGTH_LONG).show();
					  }
				  } else if (dir== 3) { // delete
					  database.delete(tableName, "RoutineID=? AND Seq=?",
					          new String[] {rid, sequence});
					  
				  }


				  database.close();
				  refreshList();
			  }
		  }
		  default:
			  break;
		  }
	  }

	  
	  @Override
	    public boolean onCreateOptionsMenu(Menu menu) {
	        MenuInflater inflater = getMenuInflater();
	        inflater.inflate(R.menu.config_menu, menu);
	        return true;
	    }
	  
	    @Override
	    public boolean onOptionsItemSelected(MenuItem item) {
	    	Intent serverIntent;
	        switch (item.getItemId()) {
	        case R.id.timing:
	        	serverIntent = new Intent(this, Timing.class);
	        	serverIntent.putExtra("RoutineID", RoutineID);
	            startActivityForResult(serverIntent, 1);
	            return true;
	        case R.id.instruction:
	        	serverIntent = new Intent(this, Instruction.class);
	        	serverIntent.putExtra("RoutineID", RoutineID);
	            startActivityForResult(serverIntent, 1);
	            return true;
	        }
	        return false;
	    }
}
